function price = BDT_up_and_in_credit(coupon_times, coupons, ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T, s, prob, delta, K, H, w, barrier_type)
% funkce pocita barierove 'up' opce na (kuponovy) bond s defaultem, kde risk-free sazba je modelovana BDT modelem
% funkce tedy umi up-and-in a up-and-out
% predpoklad modelu je, ze P_d (hodnota defaultovatelneho bondu v default stavu) < H, neboli vsechny hodnoty P na plose s defaultem jsou nize nez bariera
% funkce pracuje tak, ze pro kazdy (!) DEP vytvori aktivovany a neaktivovany strom (tim popiseme stavy na default plosine) a pak aktivovany a neaktivovany strom pro nezdefaultovane stavy
% podkladovy bond muze byt kuponovy. Bond maturuje v case s a opce na nej v case T<=s

%% vstupy
% coupon_times      = casy, kdy bond vyplaci kupony. Pokud [], tak se zadne kupony neplati a mame zero bond
% coupons           = cash kupony, ktere bond vyplaci. Neni to kuponova mira, ale fyzicky kupony, ktere chodi v case coupon times
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu pri frekvenci freq_discount. Na tyto se kalibruje BDT. Je potreba mit v tomto vektoru sazby delsi, nez je splatnost bondu v podkladu
% ycrv_maturities   = splatnosti pro ycrv_rates
% sigma             = parametr volatility log(r) v BDT modelu
% dt                = casova diference pro BDT model mela by byt takova, aby T/dt a s/dt byla cela cisla
% T                 = splatnost opce
% s                 = splatnost bondu v podkladu
% prob              = jednoperiodicka pst default. Jedna perioda ma delku dt
% K                 = strike cena opce
% H                 = bariera, jejiz hodnota musi byt > P_default
% w                 = identifikator call (w = 1) nebo put (w = -1) opce
% barrier_type      = typ bariery. (barrier_type = 1) znamena up-and-out opce, (barrier_type = -1) znamena up-and-in opci

%% vypocet
Ns = round(s/dt);
NT = round(T/dt);

%% pripava kuponu

% nejprve vytvorime stromy pro s-maturity bond
[~, BDT_Bs, ~, BDT_ds, i_grid_s, j_grid_s] = BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, s);
BDT_P = BDT_Bs.*repmat((1-prob).^(Ns - i_grid_s(:)') + delta * (1-(1-prob).^(Ns - i_grid_s(:)')), numel(j_grid_s), 1);
for k = 0:Ns
    % mame k bucketu, k = 1, ..., Ns
    % bucket k sbira cash-flow z intervalu [t_k, t_{k+1}) a cash-flow v nem se chova jako zero-bond s maturitou t_k
    % pro kazdy tento zero bond (bucket) vytvorime strom maturujici v case t_k a upravime ho o pst defaultu: hodnota na stromu * (1-pi)^(i - k), i = 0, ..., k
    % a tento strom pak 'pricteme' ke stromu zakladniho bondu P => tim ziskame strom bondu P s kupony, kde recovery se plati jen z P a ne z kuponu
    bucket_low_T = k*dt;
    bucket_high_T = (k+1)*dt;
    zz = ((coupon_times >= bucket_low_T) .* (coupon_times < bucket_high_T));
    bucket = sum(coupons(zz == 1));
     
    [~, BDT_coupon, ~, ~, ~, ~] = BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, k*dt);
    idx_s = (BDT_ds(:, k+1)>0); min_idx_s = find(idx_s == 1, 1 , 'first'); max_idx_s = find(idx_s == 1, 1 , 'last');
    BDT_coupon_tree = bucket*BDT_coupon.*repmat((1-prob).^(k - i_grid_s(1:k+1)), size(BDT_coupon, 1), 1); % uprava bucketu o pst defaultu (resp. survivalu)
    
    % pridavani 'stromu' pro 
    BDT_P(min_idx_s:max_idx_s, 1:k+1) = BDT_P(min_idx_s:max_idx_s, 1:k+1) + BDT_coupon_tree(:, 1:k+1);
end

% vytvorime strom rf bondu se splatnosti s>=T
[~, BDT_B, ~, BDT_d, i_grid, j_grid] = BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, s);
idx = (BDT_d(:, NT+1)>0); min_idx = find(idx == 1, 1 , 'first'); max_idx = find(idx == 1, 1 , 'last');
BDT_P = BDT_P(min_idx:max_idx, 1:NT+1); % omezeni stromu pro P na nezdefaultovane plose se splatnosti s na strom se splatnosti T (coz je dulezite pro opci)
BDT_B = BDT_B(min_idx:max_idx, 1:NT+1);
BDT_d = BDT_d(min_idx:max_idx, 1:NT+1);
i_grid = i_grid(1:NT+1);
j_grid = j_grid(min_idx:max_idx);
nonzero_idx = BDT_d>0;

BDT_delta = BDT_B*delta;
V_delta = get_V_delta(BDT_delta, BDT_d, K, w); % hodnota opce na zdefultovanem povrchu

k = 0;
for ii = 2:numel(i_grid) % hledame jednotlive DEP body (ii,jj (i),(j) je fyzicky (teoreticky index) DEP bodu)
   for jj = 1:numel(j_grid)
       i = i_grid(ii);
       j = j_grid(jj);
       
       if (nonzero_idx(jj, ii) > 0) % jedna se DEP
           [DEP_activated_tree, DEP_inactivated_tree] = get_DEP_tree(i, j, ii, jj, BDT_P, BDT_d, V_delta, prob, H, barrier_type);
           k = k + 1;
           
           DEP(k).k = k; DEP(k).i = i; DEP(k).j = j;
           DEP(k).DEP_activated_tree = DEP_activated_tree;
           DEP(k).DEP_inactivated_tree = DEP_inactivated_tree;
           DEP(k).price = DEP_inactivated_tree(BDT_d(:,1)>0);
       end
       
   end
end

%% jdeme resit opci na survival surface
idx = (BDT_d(:, NT+1)>0);
V_activated = 0*BDT_P;
V_inactivated = 0*BDT_P;
V_activated(idx, end) = (1-prob)^NT*max(w*(BDT_P(idx, end) - K), 0);
for ii = NT:-1:1
    nonzero_idces = BDT_d(:, ii)>0;
    for jj = 1:numel(nonzero_idces)
        if (nonzero_idces(jj) > 0)
            V_activated(jj,ii) = 0.5*BDT_d(jj, ii)*( V_activated(jj+1,ii+1) + V_activated(jj-1,ii+1));
        end
    end
end

if (barrier_type == -1); V_inactivated(:,end) = (BDT_P(:, end)>H).*V_activated(:, end); end;
if (barrier_type == 1);  V_inactivated(:,end) = (BDT_P(:, end)<H).*V_activated(:, end); end;

for ii = NT:-1:1
    nonzero_idces = BDT_d(:, ii)>0;
    for jj = 1:numel(nonzero_idces)
        if (nonzero_idces(jj) > 0 && barrier_type == -1) % up-and-in
            V_inactivated(jj,ii) = (BDT_P(jj, ii) >= H)*V_activated(jj,ii) +(BDT_P(jj, ii) < H)*0.5*BDT_d(jj, ii)*(V_inactivated(jj+1,ii+1) + V_inactivated(jj-1,ii+1));
        end
        if (nonzero_idces(jj) > 0 && barrier_type == 1) % up-and-out
            V_inactivated(jj,ii) = (BDT_P(jj, ii) < H)*0.5*BDT_d(jj, ii)*(V_inactivated(jj+1,ii+1) + V_inactivated(jj-1,ii+1));
        end
    end
end
mm = sum([DEP.price]);
price = V_inactivated(BDT_d(:,1)>0) + sum([DEP.price]);

%% funkce primo vyuziva funkci
% BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T)

%% funkce neprimo vyuziva funkci
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% get_r(T, ycrv_rates, ycrv_maturities, freq_discount_in, freq_discount_out)
end

function [DEP_activated_tree, DEP_inactivated_tree] = get_DEP_tree(i_star, j_star, ii_star, jj_star, BDT_P, BDT_d, V_delta, prob, H, barrier_type)
% funkce dela stromy (aktivovany a neaktivovany) pro DEP 
DEP_activated_tree   = 0*BDT_d;
DEP_inactivated_tree = 0*BDT_d;

%% zhotovime 'aktivovany' strom hodnot pro DEP
kappa_value = (1-prob)^(i_star-1)*prob*V_delta(jj_star, ii_star);
DEP_activated_tree(jj_star, ii_star) = kappa_value;
for ii = ii_star-1:-1:1
    nonzero_idces = BDT_d(:, ii)>0;
    for jj = 1:numel(nonzero_idces)
        if (nonzero_idces(jj) > 0)
            DEP_activated_tree(jj,ii) = 0.5*BDT_d(jj, ii)*( DEP_activated_tree(jj+1,ii+1) + DEP_activated_tree(jj-1,ii+1));
        end
    end
end

if (barrier_type == -1) 
%% bariera in
for ii = ii_star-1:-1:1
    nonzero_idces = BDT_d(:, ii)>0;
    for jj = 1:numel(nonzero_idces)
        if (nonzero_idces(jj) > 0)
            DEP_inactivated_tree(jj,ii) = (BDT_P(jj, ii) >= H)*DEP_activated_tree(jj,ii) + (BDT_P(jj, ii) < H)*0.5*BDT_d(jj, ii)*( DEP_inactivated_tree(jj+1,ii+1) + DEP_inactivated_tree(jj-1,ii+1));
        end
    end
end
end

if (barrier_type == 1)
%% bariera out
DEP_inactivated_tree(jj_star, ii_star) = kappa_value;
for ii = ii_star-1:-1:1
    nonzero_idces = BDT_d(:, ii)>0;
    for jj = 1:numel(nonzero_idces)
        if (nonzero_idces(jj) > 0)
            DEP_inactivated_tree(jj,ii) = (BDT_P(jj, ii) < H)*0.5*BDT_d(jj, ii)*( DEP_inactivated_tree(jj+1,ii+1) + DEP_inactivated_tree(jj-1,ii+1));
        end
    end
end
end

end

function V_delta = get_V_delta(BDT_delta, BDT_d, K, w)
% fce vraci strom hodnot opce na zdefaultovane plose

V_delta = 0*BDT_d;
nonzero_idces = BDT_d(:, end)>0;
V_delta(nonzero_idces, end) = max(w*(BDT_delta(nonzero_idces, end)-K), 0);

[~, ii_dim] = size(BDT_d);

for ii = ii_dim - 1:-1:1
    nonzero_idces = BDT_d(:, ii)>0;
    for jj = 1:numel(nonzero_idces)
        if (nonzero_idces(jj) > 0)
            V_delta(jj,ii) = 0.5*BDT_d(jj, ii)*( V_delta(jj+1,ii+1) + V_delta(jj-1,ii+1));
        end
    end
    
end

end