function [params, model_spread, err] = CDO_calib(tranches, quoted_tranche_spreads, properties, index_quote, coupon_times, n, delta, r)
% funkce kalibruje model (tj. parametry faktorove funkce) na kotace spreadu pro ruzne transe CDO
% kalibraci se rozumi najit takove parametry, aby modelova premia CDO transi odpovidala kotovanym
% hlavnim vystupem je vektor parametru params, kde params(1) je vzdy korelace (u Archimedan copuly je to theta) a params(2), params(3),... jsou pripade dalsi parametry
% funkce pracuje s modely: 'Normal' (jen korelace), 'Normal_stochastic' (korelace, p,q), 'NIG' (korelace, alpha), 'NIG_ext' (korelace, alpha, beta),
%                          'double_t' (korelace, nu), 'double_t_ext' (korelace, nu_X, nu_Y)
% pres systematicky faktor se integruje gaussovskou kvadraturou typu Gauss-Legendre
% kriterium optimality je minimalizace sumy ctvercu rozdilu kotovanych a modelovych premii

%% vstupy
% tranches               = matice s attachment a detachment pointy transi. Ma 2 sloupce [att1 det1; att2 det2; atd]
% quoted_tranche_spreads = hodnota kotovanych premii pro transe. Vsechna premia musi byt kotovana jako spread, ne upfront. (Pro konverzi z upfrontu do spreadu pouzit fci fair_spread)
% properties             = objekt (calass) s nastavenim rozdeleni a integrace. Musi obsahovat nasledujici promenne
    % .name     = model, ktery bude kalibrovan: 'Normal', 'NIG', 'double_t', 'Normal_stochastic'
    % .xMin     = dolni integral integrace systematickeho faktoru X
    % .xMax     = horni integral integrace systematickeho faktoru X
    % .nPoints  = pocet bodu integrace
% index quote            = kotace indexu dane iTraxx/CDX serie 
% coupon_times           = vektor casu kuponu CDO. Kazde CDO (i transe deal) ma fixni maturitu
% n                      = pocet CDO jmen v kosi (vetsinou 125)
% delta                  = recovery kazde entity v kosi
% r                      = prumerna risk-free sazba pro diskont cash-flow v dealu

%% vypocet
% optimalizacni funkce
f = @(params)(optim_fun(params, tranches, quoted_tranche_spreads, properties, index_quote, coupon_times, n, delta, r));

% volani optimalizace dle vybraneho modelu daneho v properties.name. params(1) j vzdy parametr korelace (ten se vyskytuje u vsech modelu)
if strcmp(properties.name, 'Normal')
    params = fmincon(f, 0.6, [],[],[],[], 0.001, 0.999);
elseif strcmp(properties.name, 'NIG')
    params = fmincon(f, [0.5775 0.6912], [],[],[],[], [0.001 0.0001], [0.999 20]);
elseif strcmp(properties.name, 'NIG_ext')
    A = [0 -1 -1; -1 0 0; 1 0 0; 0 -1 0; 0 1 0; 0 0 1]; b = [0; 0; 0.95;0; 3; 0];
    params = fmincon(f, [0.5775 0.6912 -0.1], A, b);%, [0.001 0.0001 -2], [0.999 20 2], @constr_NIG_ext
elseif strcmp(properties.name, 'double_t')
    params = fmincon(f, [0.5780 2.6393], [],[],[],[], [0.001 2], [0.999 40]);
elseif strcmp(properties.name, 'double_t_ext')
      params = fmincon(f, [0.95 2.05 2.2], [],[],[],[], [0.2 2.00005 2.0005], [0.999 2.1 40], [], optimoptions('fmincon','UseParallel',true));
elseif strcmp(properties.name, 'Normal_stochastic')
    params = fmincon(f, [0.5780 0.2 0.25], [],[],[],[], [0.001 0 0], [0.999 1 1]);
elseif strcmp(properties.name, 'Clayton')
    params = fmincon(f, 1.2, [],[],[],[], 0.01, 2);
elseif strcmp(properties.name, 'Gumbel')
    params = fmincon(f, 1.2, [],[],[],[], 1.01, 2.65);
elseif strcmp(properties.name, 'Gamma_exponential')
    params = fmincon(f, [0.7 0.6 0.6], [],[],[],[], [0.001 0.0001 0.00001], [0.999 inf inf]);
end

% s optimalnimi parametry zjistime celkovou chybu err a modelove spready 
[err, model_spread] = optim_fun(params, tranches, quoted_tranche_spreads, properties, index_quote, coupon_times, n, delta, r);

% vykreslime graf
scatter(quoted_tranche_spreads, model_spread); xlabel('quoted spreads');ylabel('model spreads'); grid on; hold on; plot(0:0.001:0.25, 0:0.001:0.25, 'r--'); hold off;

%% funkce primo vyuziva funkci 
% CDO_price(tranche, properties, index_quote, coupon_times, rho, n, delta, r)

%% funkce neprimo vyuziva funkci 
% lgwt(N,a,b)
% get_expected_tranche_principal(tranche, properties, x, lambda, t, rho, n, delta)
% NIG_PDF(x, alpha, beta, mu, delta)
% get_copula_prob(properties, x, lambda, t, rho)
% NIG_quantile(p, alpha, beta, mu, delta)
% NIG_CDF(x, alpha, beta, mu, delta)
% double_t_quantile(p, nu, rho)
end


function [err, model_spread] = optim_fun(params, tranches, quoted_tranche_spreads, properties, index_quote, coupon_times, n, delta, r)
% optimalizacni objektivni fce pro hledani parametru
% funkci pocita sumu ctvercu rozdilu kotovanych a modelovych premii a modelova premia pri techto parametrech
% vstupy jsou v podstate stejne jako u CDO_calib, jen jsou ve vektoru params iterovane parametry
rho = params(1);
if strcmp(properties.name, 'NIG')
    properties.NIG_alpha = params(2);
elseif strcmp(properties.name, 'NIG_ext')
    properties.NIG_alpha = params(2);
    properties.NIG_beta = params(3);
    properties.xMin = properties.NIG_beta-10;
    properties.xMax = properties.NIG_beta+10;
elseif strcmp(properties.name, 'double_t')
    properties.double_t_nu = params(2);
elseif strcmp(properties.name, 'double_t_ext')
    properties.double_t_nu = params(2);
    properties.double_t_nuY = params(3);
elseif strcmp(properties.name, 'Normal_stochastic')
    properties.normal_stochastic_p = params(2);
    properties.normal_stochastic_p = params(3);
elseif strcmp(properties.name, 'Clayton')
    properties.clayton_theta = params(1);
    properties.xMin = 0;
elseif strcmp(properties.name, 'Gumbel')
    properties.gumbel_theta = params(1);
    properties.xMin = 0;
elseif strcmp(properties.name, 'Gamma_exponential')
    properties.gamma_exponential_alpha = params(2);
    properties.gamma_exponential_beta = params(3);
end

for i = 1:numel(quoted_tranche_spreads)
    [model_spread(i), ~, ~, ~] = CDO_price(tranches(i, :), properties, index_quote, coupon_times, rho, n, delta, r);
end
model_spread = model_spread(:);
err = sum( (model_spread - quoted_tranche_spreads).^2)*10000;
[params err]
end