function [s, A, B, C] = CDO_price(tranche, properties, index_quote, coupon_times, rho, n, delta, r)
% funkce pocita fair spread s = C/(A+B), koeficient plateb premia A, AUV B, a ztrat C u transe CDO
% funkce dokaze pracovat s ruznynmi definiciemi faktorove funkce (tj. ruznych copula modelu): 'Normal' (Gaussian copula), 'NIG', 'double_t', 'Normal_stochastic'

%% poznamky (!)
% (i) CDO deal na 5Y transi ma kupony jen do predem definovae maturity transe teto serie. 
%     Napr. serie iTraxx 5Y S25 ma datum spusteni (effective date) 21.3.2016 a maturitu 20.6.2021 (tj. 21.3.2016 + 5Y + IMM) a pokud vstoupim do single tranche dealu napr.
%     v roce 2018, tak tento deal stejne bude maturovat v den 20.6.2021 bez ohledu na to, ze se jedna o 5Y variantu. Tj. maturita je ABSOLUTNE definovana, NE RELATIVNE vzhledem k deal date.
%     v tomto smyslu musi byt definovany coupon_times, tj. musi brat v uvahu, ze maturita CDO dealu je fixni
% (ii) dalsi vlastnosti je, ze ve chvili, kdy vstoupim do dealu, tak je jmenny seznam creditu v kosi jakoby zafixovan. 
%     Tj. i kdyz je treba pak jmeno z kose vyhozeno, tak default ovlivnuje transi! 
% (iii) idealni vstupy lze naloadovat ze souboru CDO_data.mat
% (iv) pro vyber modelu je dobre projit tridu properties (tam se nastavuje copula, rozsah integrace pres X atd...)
% (v) long CDO je protection seller, tj. plati ztraty v transi, zpusobene defaulty v portfoliu. short CDO je protection buyer, tj. plati premium a dostava jednorazove platby z defaultu
% (vi) upfront: (u > 0) -> short CDO (p. buyer) posila abs(u) long CDO (p. sellerovi);  (u < 0) -> long CDO (p. seller) posila abs(u) short CDO (p. buyerovi). u = C*(1-s_star/s);

%% vstupy
% tranche       = vektor dvou hodnot [attachment detachment] transe, pro kterou hledame fair spread
% properties    = objekt (calass) s nastavenim rozdeleni a integrace. Musi obsahovat nasledujici promenne
    % .xMin         = dolni integral integrace systematickeho faktoru X
    % .xMax         = horni integral integrace systematickeho faktoru X
    % .nPoints      = pocet bodu integrace
    % .name         = charakterizace faktorove fce: 'Normal' (Gaussian copula), 'NIG', 'double_t', 'Normal_stochastic'
    % .NIG_alpha    = parametr alpha u NIG rozdeleni (NIG specifikace)
    % .double_t_nu  = parametr stupnu volnosti u t rozdeleni (double_t specifikace)  
    % .normal_stochastic_p = parametr p u normal_stochastic
    % .normal_stochastic_q = parametr q u normal_stochastic
% index quote   = kotace indexu dane iTraxx/CDX serie 
% coupon_times  = vektor casu kuponu CDO. Kazde CDO (i transe deal) ma fixni maturitu
% rho           = copula korelace
% n             = pocet CDO jmen v kosi (vetsinou 125)
% delta         = recovery kazde entity v kosi
% r             = prumerna risk-free sazba pro diskont cash-flow v dealu

%% priklad volani:
% load('CDO_data.mat')
% [s, A, B, C] = CDO_price(CDO_data.iTraxx.S10_5Y_01_10_2008.tranches(1,:), properties, CDO_data.iTraxx.S10_5Y_01_10_2008.index_quote, CDO_data.iTraxx.S10_5Y_01_10_2008.coupon_times, ...
% 0.55, 125, 0.4, CDO_data.iTraxx.S10_5Y_01_10_2008.r)


%% vypocet
[A, B, C] = get_ABC_uncond(tranche, properties, index_quote/(1-delta), coupon_times, rho, n, delta, r);
s = C/(A+B);

%% funkce primo vyuziva funkci 
% lgwt(N,a,b)
% get_expected_tranche_principal(tranche, properties, x, lambda, t, rho, n, delta)
% NIG_PDF(x, alpha, beta, mu, delta)

%% funkce neprimo vyuziva funkci
% get_copula_prob(properties, x, lambda, t, rho)
% NIG_quantile(p, alpha, beta, mu, delta)
% NIG_CDF(x, alpha, beta, mu, delta)
% double_t_quantile(p, nu, rho)
end


function [A, B, C] = get_ABC_uncond(tranche, properties, lambda, coupon_times, rho, n, delta, r)
% funkce ziskavajici komponenty A, B, C, ze kterych se pocita fair spread jako s = C/(A+B)
coupon_times = coupon_times(:)';
yearfracs = [coupon_times(1) diff(coupon_times)]; % jednotlive yearfractions, ktere se aplikuji na vysi kuponu

% mid casy (resp. intervaly) se aproximuje nejaky integral
mid_coupon_times = 0.5*([0 coupon_times(1:end-1)]+coupon_times(1:end));

% hodnota 'nahodne promenne' X, pres jejiz hustotu budeme integrovat a integracni vahy pro Gaussovskou kvadraturu
[x, w] = lgwt(properties.nPoints,properties.xMin,properties.xMax);
E_integrand = get_expected_tranche_principal(tranche, properties, x, lambda, coupon_times, rho, n, delta);

% ziskani fx = PDF(x), tj. funkci hustoty faktoru X
if (strcmp(properties.name, 'Normal') || strcmp(properties.name, 'Normal_stochastic') || strcmp(properties.name,'Gamma_exponential'))
    PDF_x = normpdf(x);
elseif strcmp(properties.name, 'NIG')
    PDF_x = NIG_PDF(x, properties.NIG_alpha, 0, 0, properties.NIG_alpha);
elseif strcmp(properties.name, 'NIG_ext')
    alpha = properties.NIG_alpha; beta = properties.NIG_beta; gam = sqrt(alpha^2 - beta^2);  
    PDF_x = NIG_PDF(x, alpha, beta, -beta*gam^2/alpha^2, gam^3/alpha^2);
elseif strcmp(properties.name, 'double_t')
    PDF_x = tpdf(x, properties.double_t_nu);
elseif strcmp(properties.name, 'double_t_ext')
    PDF_x = tpdf(x, properties.double_t_nu);
elseif strcmp(properties.name, 'Clayton')
    PDF_x = gampdf(x, 1.0/properties.clayton_theta, 1.0);
elseif strcmp(properties.name, 'Gumbel')
    theta = properties.gumbel_theta;
    PDF_x = stable_PDF(x, 1/theta, 1, cos(pi/(2*theta))^theta, 0);
end

% vypocet komponent A (kupony), B (AUV), C (platby za ztraty)
A = sum(w.*PDF_x(:).*sum(E_integrand(:, 2:end).*repmat(exp(-r*coupon_times).*yearfracs, numel(x), 1), 2));
B = 0.5*sum(w.*PDF_x(:).*sum((E_integrand(:, 1:end-1) - E_integrand(:, 2:end)).*repmat(exp(-r*mid_coupon_times) .*yearfracs, numel(x), 1), 2)) ;
C = sum(w.*PDF_x(:).*sum((E_integrand(:, 1:end-1) - E_integrand(:, 2:end)).*repmat(exp(-r*mid_coupon_times), numel(x), 1), 2)) ;

end