function [An, P] = loss_distrib_sector(exposures, PD, PD_sigma, LGD, unit)
% funkce je implementaci modelu CreditRisk+ s variabilnimi default raty, kde vsechny expozice jsou v jednom sektoru
% vystupem je An = vektor pst ztrat a P vektor psti poctu defaultu

%% vstupy
% exposures = vektor expozic (nijak neupravenych o LGD) a v peneznich jednotkach, ne v unitech
% PD = psti defaultu
% PD_sigma = volatilita PD
% LGD = LGD, ktere se aplikuje na expozice, aby se ziskala jen cast, kterou ztratime pri defaultu
% unit = pocet jednotek, ve kterych se bude vyjadrovat ztrata

%% vypocet
% ***** zakladni transformace vstupnich dat na modelova data **********
% pripravi se (i) nu = jakasi zaokrouhlena defaultovatelna expozice v jednotkach unit a (ii) epsilon, ocekavana ztrata v jednotkach unit
nu = ceil(exposures.*LGD/unit);    % pocet jednotek ztraty v pripade defaultu (tj. 'defaultovatelna expozice') (zaokrouhleno na cela cisla)
                                   % toto cislo (nu) je v technickem dokumentu oznaceno jen jako pocet jednotek 'expozice' (eq. (11))
                                   % maximalni dosazitelna ztrata v portfoliu (v jednotkach) je tedy sum(nu)
epsilon = (PD.*LGD.*exposures)/unit; % ocekavana ztrata vyjadrena v jednotkach (nezaokrouhluje se)
% **********************************************************************

% ********************** priprava dat pro 'expozicni' pasma dle vzorce (12) ********************
% vytvoreni expzocnich pasem j = 1, ..., m
% nu_band = distinct nu (tj. vsechny mozne expozice)
% epsilon_band = suma epsilonu v jednotlivych expozicnich pasmech
[nu_band, epsilon_band, ~] = exposure_bands(nu, epsilon);

% mu = je PD ziskane ze zaokrouhlenych 'defaultovatelnych expozic' nu
mu = epsilon./nu;
sigma_round = PD_sigma.*mu./PD; % ziskani zaokrouhlenuch sigma_PD. Neni v technickem dokumentu, ale vypocty pak sedi presne na CR+ demo excel

% parametry sektoru (mu_k, sigma_k)
mu_k = sum(mu); % dle vzorce (39), kde mu je epsilon/nu
sigma_k = sum(sigma_round); % dle vzorce (42), kde sigma_A je sigma_round

% dalsi parametry sektoru pro negativni binomicke rozdeleni
alpha_k = (mu_k/sigma_k)^2;
beta_k = (sigma_k^2)/mu_k;
p_k = beta_k/(1+beta_k);

%%% hlavni vypocetni cast
% ******************* konstrukce polynomu A a B dle vzorce (80) *******************************
A_poly = [];
B_poly = [];
for j = 1:numel(nu_band); A_poly(nu_band(j)-1 +1) = epsilon_band(j); end;
A_poly = p_k*alpha_k/mu_k*A_poly;

for j = 1:numel(nu_band); B_poly(nu_band(j) +1) = epsilon_band(j)/nu_band(j); end;
B_poly = -p_k/mu_k*B_poly; B_poly(1) = 1;
% ********************************************************************************************

% zjisteni hodnot r, s dle dimenze polynomu (tvar polynomu dle vzorce (71))
r = numel(A_poly)-1;
s = numel(B_poly)-1;

% ****************** reseni rekurze pro vektor An dle vzorce (72) **************************
% An(n +1) = Pr[ztrata portfolia = n*unit]

An(0 +1) = (1-p_k)^alpha_k; %pst, ze ztrata je 0, to odpovida psti 0 defaultu. Tu vezmeme ze vzroce (57)
for n = 1:sum(nu*1.5)
    % rekurze dle vzorce (72)
    % vzorec jsem jen upravil tak, aby rekurze nebyla pro (n+1), ale pro (n). Tj. v (72) jsem misto (n) pouzil (n-1)
    sum1 = 0;
    sum2 = 0;

    for i = 0:min(r, n-1); sum1 = sum1 + A_poly(i +1)*An(n-i-1 +1); end;
    for j = 0:min(s-1, n-2); sum2 = sum2+B_poly(j+1 +1)*(n-j-1)*An(n-j-1 +1); end;
    
    An(n +1) = 1/(B_poly(0 +1)*n)*(sum1 - sum2);
end
An = An(:);

% vypocet vektor psti defaultu P(i +1) = Pr[pocet defaultu = i] dle vzorce (57)
% pocet defaultu v portfoliu ma negativni binomicke rozdeleni
%for i = 0:numel(nu); P(i +1) = ((1-p_k)^alpha_k)*(p_k^i)*nchoosek(i+alpha_k -1, i); end; P = P(:);
P = [];
%% funkce primo vyuziva funkci 
% exposure_bands(nu, epsilon)

%% funkce neprimo vyuziva funkci
% --
end