function [Gx] = get_loss_vec_rnd_RR(E, x, Q_mat, rho_RR, alpha_RR, beta_RR, unit, cluster_size)
% funkce vraci koeficienty loss-PGF G(z|X=x) pro cele portfolio a kazdou hodnotu systemove promenne X. Vysledkem je matice, kde radky jsou hodnoty G pro X=x_k
% predpoklada se, ze RR jsou stochasticke a pouziva se (pro PD i RR) Gaussian copula 

%% vstupy
% E             = vektor expozic (nijak neupravenych) v beznych peneznich jednotkach
% x             = vektor hodnot systemoveho faktoru X=x
% Q_mat         = matice Q_i(x), kde radek je Q_i(X=x), i = 1, 2, ..., n; tj. vektor psti defaultu vsech dluzniku v portfoliu given X=x
% rho_RR        = vektor korelaci RR dluzniku se systemovym faktorem X
% alpha_RR      = vektor parametru alpha (pro kazdeho dluznika jeden) charakterizujici beta rozdeleni jeho RR
% beta_RR       = vektor parametru beta (pro kazdeho dluznika jeden) charakterizujici beta rozdeleni jeho RR
% unit          = jednotka zuctovani
% cluster_size  = pokud 0, tak se loss-PGF pocita beznou FFT metodou. Poud (integer) > 0, tak se pocita clustered FFT metodou

% POZOR funkce neumi resit pripad, kdy by 2 a vice recovery cohorty generovaly stejnou ztratu. To zpravidla nastane, pokud je unit velke cislo nebo thresholdy moc u sebe
% napriklad: mame expozici 3 a recovery delta_j = [0.05, 0.1, ...]. Pak ztraty odpovidajici temto recovery bude ceil((1-0.05)*3) = 3, ceil((1-0.1)*3) = 3, ...
% s timto si funkce neumi poradit a predpoklada, ze ztraty v ruznych recovery kohortach jsou ruzna cisla 

%% vypocet
maxLoss = sum(ceil(E(:)/unit)); % maximalni ztrata dosazitelna na portfoliu. Predpokladame, ze odpovida cele expozici, protoze s nejakou (napr. 0 pst) muze ztratit vse
Gx = zeros(size(Q_mat, 1), maxLoss+1); % predalokace matice loss-generating function. k_ty radek drzi loss-generating vector given X=x_k

[RR_thresholds, delta, beta_probs] = prepare_RR_thresholds(alpha_RR, beta_RR); % nacteme hodnoty RR thresholdu a delta_j, ktere jsou mozne pro jednotlive dluzniky
FZi_inv = norminv(beta_probs); % inverzni fce kvantilu RR thresholdu, zustava stejna v celem vypoctu, proto se predpocitava 

losses = ceil(repmat(1-delta(:)', numel(E), 1).*repmat(E(:), 1, numel(delta))/unit); % ztraty, v jednotkach unit, ktere budou u danych expozic realizovany

if (cluster_size == 0) % bezna FFT metoa (bez clusteringu)
    row_idx = repmat((1:numel(E))', numel(delta), 1); % vektor s opakovanim i = 1, 2,..., n, 1, 2, ..., n; slouzi k namapovani psti ztrat do radku k dluznikum
    idx = sub2ind([numel(E), maxLoss+1], row_idx, losses(:)+1); % najdeme indexy, do kterych se priradi psti ztraty jednotlich dluzniku, given X=x_k
    for k = 1:numel(x) % iterujeme pres vsechny hodnoty X=x=x_k
        [probs] = get_prob_RR(x(k), rho_RR, RR_thresholds, FZi_inv); %psti Pr[delta_i = delta_j|X=x_k], radky znaci jednotlive dluzniky
        loss_mat = zeros(numel(E), maxLoss+1); % matice s loss-generating vectory vsech jmen na portfoliu
        Q_row = Q_mat(k, :); % psti defaultu pro vsechny jmena v portfoliu, given X=x_k  
        loss_mat(:, 1) =  1-Q_row(:); % vektor psti jednotlivych dluzniku, ze nebude default, given X=x_k priradime do matice ztrat given X=x_k
        RR_probs_default_probs = probs.*repmat(Q_row(:), 1, numel(delta)); % matice s psti ztraty*defaultu

        loss_mat(idx) = loss_mat(idx(:))+ RR_probs_default_probs(:); % vlozeni psti ztrat to matice s psti ztraty

        Gx(k,:) = ifft(prod(fft(loss_mat, [], 2), 1)); % spocitame agregovany loss-generating vector pres cele portfolio, given X=x_k 
    end
else % clustered FFT
    num_clusters = ceil(numel(E)/cluster_size); % pocet clusteru, do kterych bude portfolio rozdeleno
    for k = 1:size(Q_mat, 1) % iterujeme pres vsechny hodnoty X=x=x_k
        Gx_cluster = zeros(num_clusters, sum(ceil(E(:)/unit))+1); % loss-PGF pro vsechny clustery. Jedna se o matici, kde pocet radku = pocet clusteru
        [probs] = get_prob_RR(x(k), rho_RR, RR_thresholds, FZi_inv); % psti Pr[delta_i = delta_j|X=x_k], radky znaci jednotlive dluzniky
        for cluster = 1:num_clusters % pro dane X=x_k jdeme pres vsechny clustery
            cluster_idces = (cluster-1)*cluster_size+1:min(cluster*cluster_size, numel(E)); % indexy dluzniku v clusteru
            cluster_Q_row = Q_mat(k,cluster_idces); % X=x_k--podminene psti defaultu dluzniku v clusteru
            losses_cluster = losses(cluster_idces, :); % ztraty, ktere mohou v clusteru nastat (jedna se o matici, kde radky jsou dluznici a sloupce hodnoty ztrat)
            maxLoss = sum(losses_cluster(:,1)); % maximalni ztrata v clusteru
            loss_mat = zeros(numel(cluster_idces), maxLoss+1); % matice s psti ztrat v loss_mat (given default) v danem clusteru
            loss_mat(:, 1) =  1-cluster_Q_row(:); % psti nedefaultu znamenaji 0 ztratu. Vlozime je do matice s psti ztrat jako prvni
            idx = sub2ind(size(loss_mat), repmat( [1:size(losses_cluster, 1)]', numel(delta), 1), losses_cluster(:)+1); % dohledame indexy, pres ktere vlozime psti ztraty
            RR_probs_default_probs = probs(cluster_idces, :).*repmat(cluster_Q_row(:), 1, numel(delta)); % matice s psti ztraty*defaultu. Tj. upravime psti ztrat o pst defaultu
            loss_mat(idx(:)') = loss_mat(idx(:)')+ RR_probs_default_probs(:)'; % pridame psti ztrat do matice s pst vsech moznych ztrat dluzniku v clusteru
            Gx_cluster(cluster, 1:maxLoss+1) = ifft(prod(fft(loss_mat, [], 2), 1)); % pres FFT spocitame loss-PGF v danem clusteru
        end
        Gx(k,:) = ifft(prod(fft(Gx_cluster, [], 2), 1)); % pres FFT zagregujeme loss-PGF vsech clusteru na uroven ztrat v portfoliu (given X=x_k)
    end 
end

%% funkce primo vyuziva funkci
% prepare_RR_thresholds(alpha_RR, beta_RR)
% get_prob_RR(x, rho_RR, RR_thresholds, FZi_inv)
end