function [BDT_r, BDT_B, BDT_Q, BDT_d, i_grid, j_grid] = BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T)
% funkce vytvari BDT stromy pro sazbu, cenu zero bondu B(t,T), arrow-debreu, jednoperiodickych diskontnich faktoru. Vsechny procesy jsou od casu t = 0 do casu t = T
% vystupem jsou take teoreticke hodnoty i, j (teoreticke = hodnoty z knihy)
% urcitym omezenim modelu (ne implementace) je, ze dt je shodny pro 'tenor sazby' i casovy spacing na mrizce
% nelze pak treba namodelovat strom 3M sazby pro casove intervaly 0.1, 0.2, 0.3; ale musi se jit take po 3M

% cas T je rozsekan do N = T/dt intervalu, mame tedy N+1 bodu v case, i = 0,...,N. Cas ti = i*dt. t0 = 0*dt = 0 a tN = N*dt = T
% fyzicky index casu (v matici) tedy odpovida casu (i-1)*dt. Pokud tedy v Matlabu mame napr. bod [~, 5], tak jsme v case t = (5-1)*dt = 4*dt
%                                                            Posledni index casu v matici odpovida casu T
% r ze stromu BDT_r je kratka sazba r = r_ij = r(i*dt, (i+1)*dt) v poloze j. Poslednim r, tj. r(T, T+dt) lze diskontovat CF z casu T+dt
% B ze stromu BDT_B je strom vyvoje hodnot bondu B(t, T) v case t (T je fixovano). Posledni B ze stromu BDT_B jsou 1, protoze bond je ve splatnosti
% Q ze stromu BDT_Q je psti-vazene PV cash flow 1, ktere dostaneme, pokud nastane stav ij. PV je jakoby k casu t=0 (!). Posledni Q ze stromu BDT_Q odrazi pst. vazenou hodnotu CF z casu T k casu t=0
% d ze stromu BDT_d je jednoperiodicky diskontni faktor v case t = i*dt pro diskont CF z casu (i+1)*dt. d_ij = 1/(1+r_ij*dt). Poslednimi d ze stromu BDT_d lze tedy diskontovat CF z casu T+dt

%% vstupy
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu pri frekvenci freq_discount. Na tyto se kalibruje BDT. Je potreba mit v tomto vektoru sazby delsi, nez je splatnost BDT bondu
% ycrv_maturities   = splatnosti pro ycrv_rates
% freq_discount     = frekvence uroceni, ktera se predpoklada u kotace sazeb ycrv_rates
% sigma             = parametr volatility ln(r) u BDT modelu
% dt                = casovy spacing u BDT modelu a tez tenor kratke sazby, kterou modelujeme
% T                 = splatnost zero bondu modelovaneho BDT

%% vypocet
N = round(T/dt);
j_grid = (N:-1:-N)';
i_grid = 0:N;
tree = zeros(2*N+1, N+1);
BDT_r = tree;
BDT_Q = tree;
BDT_d = tree;

% i = 0 (inicializace v case t = 0)
r0 = get_r(dt, ycrv_rates, ycrv_maturities, 1/dt, freq_discount);
BDT_r(N+1, 1) = r0;
BDT_Q(N+1, 1) = 1;
BDT_d(N+1, 1) = 1/(1+r0*dt);

% i = 1 az N
for i = 1:N % i a j zde maji podobu teoretickych indexu (tj. indexu z textu knihy napr.)
    
    % hodnota bondu B(0, ti + dt) maturujiciho pristi periodu 
    B_ti_plus_dt = get_bond_price((i+1)*dt, ycrv_rates, ycrv_maturities, freq_discount);
    
    j_rng = i:-2:-i; % j(i) = i:-2:-i
    Qij = get_Q(i, j_grid, BDT_Q, BDT_d); % hodnoty Qij, pro cas i a vsechny j(i) = -i:2:i
    
    % zjisteni lambda = lambda(i) matchovanim ceny bondu B(0, ti + dt)
    f = @(lambda)( (sum(Qij(:)./ (1+lambda*exp(sigma*j_rng(:)*sqrt(dt))*dt)) - B_ti_plus_dt)*10000000);
    lambda = fzero(f, 0.0005);
    
    [i_physical, j_physical] = ij(i, j_rng, j_grid);
    r = lambda*exp(sigma*j_rng(:)*sqrt(dt)); % vypocet sazeb r = r_ij pro cas i a vsechny j(i) = -i:2:i
    d = 1./(1+r*dt); % vypocet jednoperiodickych diskontnich faktoru d = d_ij pozorovane v case i a vsechny j(i) = -i:2:i
    
    BDT_Q(j_physical, i_physical) = Qij;
    BDT_r(j_physical, i_physical) = r;
    BDT_d(j_physical, i_physical) = d;
end

% strom hodnot B(t,T), t = 0 az T se ziskava az na konci, kdyz mame BDT_r, BDT_Q, BDT_d
BDT_B = bond_tree(BDT_d, N, j_grid);

%% funkce primo vyuziva funkci
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
% get_r(T, ycrv_rates, ycrv_maturities, freq_discount_in, freq_discount_out)

%% funkce neprimo vyuziva funkci
% --
end

function BDT_B = bond_tree(BDT_d, N, j_grid)
% funkce ke konstrukci stromu hodnot bondu B(t,T)
BDT_B = 0*BDT_d;
BDT_B(BDT_d(:,end)>0,end) = 1; 

for i = N-1:-1:0
    j_rng = -i:2:i;
    for idx = 1:numel(j_rng)
       j = j_rng(idx);
       [~, j_low] = ij(i+1, j-1, j_grid);
       [i_phys, j_high] = ij(i+1, j+1, j_grid);
       B_low = BDT_B(j_low, i_phys);
       B_high = BDT_B(j_high, i_phys);
       [i_phys, j_phys] = ij(i, j, j_grid);
       BDT_B(j_phys, i_phys) = BDT_d(j_phys, i_phys)*(0.5*B_low + 0.5*B_high);
    end
end

end

function Qij = get_Q(i, j_grid, BDT_Q, BDT_d)
% funkce ziskava Qij, pro i dane a vsechna j(i) = -i:2:i
j_rng = i:-2:-i;

for idx = 1:numel(j_rng)
    j = j_rng(idx);
    [~, j_low]  = ij(i-1, j-1, j_grid); [i_prev, j_high] = ij(i-1, j+1, j_grid);
    
    if (numel(j_low) > 0);  d_low = BDT_d(j_low, i_prev);    Q_low = BDT_Q(j_low, i_prev);   else d_low =0; Q_low = 0;  end;
    if (numel(j_high) > 0); d_high = BDT_d(j_high, i_prev);  Q_high = BDT_Q(j_high, i_prev); else d_high =0; Q_high = 0; end;
    
    Qij(idx) = (j==-i)*0.5*Q_high*d_high + ((j~=-i) && (j~=i))*(0.5*Q_high*d_high + 0.5*Q_low*d_low) + (j==i)*0.5*Q_low*d_low;
end

end

function [i, j] = ij(i, j, j_grid)
% funkce prevadi i, j teoreticke na i,j fyzicke. j_grid je cely rozsah j hodnot, NENI to j(i), ale j(T)
% j physical = (N+1)-j theoretical 
i = i+1;
j = find(ismember(j_grid,j));
end