function sigma = BDT_calib_to_swaptions(ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix, dt)
% funkce kalibruje BDT IR model na trzni kotace evropskych swapci
% vystupem je parametr sigma BDT modelu

% optimalizace pracuje s hledanim nejlepsiho fittu na ceny payer swapci, ktere ziskame Blackovym swapcnim modelem 
% nejlepsi fit je ve smyslu nejmensi celkove relativni chyby, tedy suma( abs(cena_BDT_model(i)/cena_Black(i) - 1) ), pres vsechna i. Kazde i je jedna swapce z vektoru (quote_ivol a quote_K)

%% vstupy
% ycrv_rates            = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities       = splatnosti pro ycrv_rates
% quote_ivol            = kotace blackovych implikovanych volatilit pro swapce. (vektor)
% quote_K               = striky swapci. (vektor)
% swaption_maturities   = expirace kotovanych swapci (v letech) 
% IRS_tenors            = tenory podkladovych IRS
% yearfrac_fix          = jakoby tenor fixni nohy IRS. Zaroven se pouziva tento yearfrac (-> frekvence uroceni) i pro spocitani diskontu ve vsech vzorcich
% dt                    = casovy interval v BDT modelu. dt by melo byt takove, aby odpovidalo tenoru 'kratke' sazby, pro kterou model kalibrujeme (typicky napr. dt = 0.25)

%% vypocet
f = @(params)(optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix, dt));
params = fmincon(f, 1.0 , [], [], [], [], 0.000001, 2.1);

[err, swaption_val, swaption_val_quote] = optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix, dt);

sigma = params;

scatter(swaption_val_quote*10000, swaption_val*10000); axis([0 150 0 150]); xlabel('swaption premia - market (bps)'); ylabel('swaption premia - BDT model (bps)'); grid on;
set(gca,'XTick',0:30:150);
set(gca,'YTick',0:30:150);
hold on;
plot(0:150, 0:150, 'r--');
hold off;
%% funkce primo vyuziva funkci 
% optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix) ... zde v m. filu
% black_swaption(K, swaption_maturity, IRS_tenor, sigma, yearfrac_fix, w_receiver_payer, ycrv_rates, ycrv_maturities)

%% funkce neprimo vyuziva funkci 
% BDT_swaption(K, swaption_maturity, IRS_tenor,sigma, ycrv_rates, ycrv_maturities, yearfrac_fix, w_receiver_payer, dt)
% BDT(ycrv_rates, ycrv_maturities, freq_discount, sigma, dt, T)
% get_fwd_swap_rate(IRS_start_T, IRS_tenor, ycrv_rates, ycrv_maturities, freq_discount)
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end

function [err, swaption_val, swaption_val_quote] = optim_fun(params, ycrv_rates, ycrv_maturities, quote_ivol, quote_K, swaption_maturities, IRS_tenors, yearfrac_fix, dt)
% objektivni funkce pro vypocet celkove relativni chyby.
% relativni chybou se rozumi odchylka cen swapci z HW modelu a kotovanych cen (tj. z Blackova modelu)

w_receiver_payer = -1; % nastavujeme, ze hledame cenu payer swapce
sigma = params;

for swaption = 1:numel(quote_ivol) % projdeme vsechny swapce v datasetu
    swaption_val(swaption) = BDT_swaption(quote_K(swaption), swaption_maturities(swaption), IRS_tenors(swaption), sigma, ycrv_rates, ycrv_maturities, yearfrac_fix, w_receiver_payer, dt);
    swaption_val_quote(swaption) = black_swaption(quote_K(swaption), swaption_maturities(swaption), IRS_tenors(swaption),quote_ivol(swaption), yearfrac_fix, w_receiver_payer, ycrv_rates, ycrv_maturities);
end

% vypocet celkove relativni odchylky cen swapci z HW modelu od cen z trhu
diffs_relative = swaption_val./swaption_val_quote-1;
err = sum(abs(diffs_relative));
end