function price = HW_swaption_price(K, swaption_maturity, IRS_tenor, kappa, sigma, ycrv_rates, ycrv_maturities, yearfrac_fix, w_receiver_payer)
% funkce pocita hodnotu swapce v Hull-Whitove IR modelu pomoci Jamshidianova triku (dekompozice vyplaty swapce na opce na zero-bondy)

%% vstupy
% K                 = strike swapce
% swaption_maturity = cas T0, tedy cas, kdy swapce expiruje a zaroven zacina bezet IRS v podkladu swapce (T0 je prvni reset floatove nohy IRS)
% IRS_tenor         = delka IRS v podkladu opce. Pozor, toto NENI CAS, kdy IRS konci, je to delka IRS
% kappa             = sila mean-reversion v HW modelu
% sigma             = parametr volatility kratke sazby r(t) v HW modelu
% ycrv_rates        = vektor urokovych sazeb pozorovanych na trhu
% ycrv_maturities   = splatnosti pro ycrv_rates
% yearfrac_fix      = delka obdobi pro uroceni. Zde ma dve role: 
%                         a) pouziva se pro vypocteni ceny bondu (pro diskontovani timto bondem v opcnim modelu)
%                         b) zaroven se vyuziva i jako yearfrac fixni nohy IRS v podkladu
%                     pozn: na Bloombergu je typicky u swapci i kotovanych sazbe predpokladano S/A uroceni, tj. yearfrac_fix by byl 0.5 
% w_receiver_payer  = 1: hodnota receiver swapce
%                    -1: hodnota payer swapce

% POZN 1: pokud ocenujeme swapci T0 x (Tn - T0), tak: swaption_maturity = T0, IRS_tenor = Tn - T0 
% POZN 2: tenor (napr 3M, 6M) floatove nohy podkladoveho IRS  je irelevantni, nikam nevstupuje 

%% vypocet
% najdeme kritickou hodnotu x_crit, pro kterou bude swapce (je jedno jestli payer nebo receiver, protoze uroven x_crit je stejna pro obe) ATM
% payer swapce je ITM v case T0, pokud x(T0)>x_crit; receiver swapce je ITM v case T0, pokud x(T0)<x_crit
f = @(x)(HW_find_x_critical(x, kappa, sigma, swaption_maturity, K, IRS_tenor, yearfrac_fix, ycrv_rates, ycrv_maturities));
x_crit = fzero(f, 0.0);

% pomoci kritickych hodnot definujeme striky, K_jamshdian pro aplikaci Jamshidianova triku
N = IRS_tenor/yearfrac_fix;  % pocet fixnich plateb kuponu IRS
Ti = swaption_maturity + (1:N)*yearfrac_fix; % casy plateb fixnich kuponu IRS

% vytvorime fiktivni striky na bond opce pomoci Jamshidianova triku. Plati, ze % K_jamshidian(i) = B(T0, Ti, x_crit)
K_jamshidian = HW_bond_reconstitution(swaption_maturity, Ti, x_crit, kappa, sigma, ycrv_rates, ycrv_maturities, 1/yearfrac_fix);

% pokud pocitame: 
%   a) hodnotu RECEIVER swapce (w =  1), tak je ekvivalentni portfoliu CALL opci na zero bondy (call_put =  1)
%   b) hodnotu PAYER swapce    (w = -1), tak je ekvivalentni portfoliu PUT opci na zero bondy  (call_put = -1)
call_put = w_receiver_payer;
for i = 1:N % vypocet opci na zero bondy se striky K_jamshidian z Jamshidianova triku
   bond_option_value(i) = HW_bond_option(swaption_maturity, Ti(i), K_jamshidian(i), ycrv_rates, ycrv_maturities, 1/yearfrac_fix, kappa, sigma, call_put); 
end

% vysumujeme hodnoty jednotlivych bond opci
price = bond_option_value(end) + K*yearfrac_fix*sum(bond_option_value);

%% funkce primo vyuziva funkci 
% HW_find_x_critical(x, kappa, sigma, swaption_maturity, K, IRS_tenor, yearfrac_fix, ycrv_rates, ycrv_maturities)...zde v m. filu
% HW_bond_option(T, s, K, ycrv_rates, ycrv_maturities, freq_discount, kappa, sigma, call_put)
% HW_bond_reconstitution(t, T, x, kappa, sigma, ycrv_rates, ycrv_maturities, freq_discount)

%% funkce neprimo vyuziva funkci 
% get_bond_price(T, ycrv_rates, ycrv_maturities, freq_discount)
end


function val = HW_find_x_critical(x, kappa, sigma, swaption_maturity, K, IRS_tenor, yearfrac_fix, ycrv_rates, ycrv_maturities)
% implementace vzorce (10.22) z Piterbarga II pro dohledani kriticke hodnoty x_crit, pro kterou bude swapce v cast T0 at-the-money
% tenor floatove nohy je irelevantni, zalezi jen na yieldfracu fixni nohy
T0 = swaption_maturity;
TN = swaption_maturity + IRS_tenor;
N = IRS_tenor/yearfrac_fix;
i = 0:N-1;
Ti_plus_1 = T0 + (i+1)*yearfrac_fix;

B_TN = HW_bond_reconstitution(T0, TN, x, kappa, sigma, ycrv_rates, ycrv_maturities, 1/yearfrac_fix);
B_Ti_plus_1 = HW_bond_reconstitution(T0, Ti_plus_1, x, kappa, sigma, ycrv_rates, ycrv_maturities, 1/yearfrac_fix);

val = B_TN + K*yearfrac_fix * sum(B_Ti_plus_1)-1;

end