function [price] = heston_trap_quad(S, K, T, r, q, v, theta, kappa, sigma, rho, w, x_quad, w_quad)
% implementace hestonova modelu Rouah Vainberg (s prihlednutim k 'trap' problemu dle Albrechera) - s vlastni numerickou integraci
% numericka integrace probiha pomoci Gauss-Laguerre kvadratury. Tato kvadratura integruje pres (0, +inf) 
% pokud bychom chteli jine typy kvadratur (napr. Gauss-Legendre), museli bychom upravit vyraz (w_quad.*f(x_quad)), ktery provadi Gauss-Laguerre integraci
% x_quad a w_quad mam v souboru (wx_gauss_laguerre.mat) v adresari PhD/quadratures
% !! FUNKCE UMI VYHODNOCOVAT vicero cen na jednou pro ruzna S, v! Staci poslat S, v jako vektory. Ostatni promenne musi zustat skalary (mimo x_quad a w_quad)

%% vstupy
% S         = spotova cena podkladoveho aktiva
% K         = strike opce
% T         = residualni doba do splatnosti opce
% r         = hodnota risk-free sazby pro splatnost T
% q         = hodnota spojite dividendy
% v         = rozptyl v case oceneni (t = 0)
% theta     = dlouhodoba uroven (mean revesion uroven) rozptylu
% kappa     = rychlost mean revesion rozptylu
% sigma     = volatilita rozptylu
% rho       = korelace mezi Wienerovskymi procesy rozptylu a ceny podkladu
% w         = identifikator call opce (w = 1) nebo put opce (w = -1)
% x_quad    = body x v kvadrature
% w_quad    = vahy w v kvadrature

%% vypocet
dim_S    = numel(S);
dim_quad = numel(x_quad);


% S, v, x_quad, w_quad budou vzdy v matici, ktera ma dim(S) radku a dim(x_quad) sloupcu
S               = repmat(S(:), 1, dim_quad); % vytvoreni matice S, kde kazdy sloupec obsahuje hodnoty S(1), ..., S(end)
v               = repmat(v(:), 1, dim_quad); % vytvoreni matice v, kde kazdy sloupec obsahuje hodnoty v(1), ..., v(end)
x_quad          = repmat(x_quad(:)', dim_S, 1); % vytvoreni matice x_quad, kde kazdy radek ma hodnoty x_quad(1), ..., x_quad(end)
w_quad          = repmat(w_quad(:)', dim_S, 1); % vytvoreni matice w_quad, kde kazdy radek ma hodnoty w_quad(1), ..., w_quad(end)

u = [0.5 -0.5];
b = [kappa - rho*sigma kappa];

%% pomalejsi implementace
% d = @(j, x_quad)( sqrt((rho*sigma*x_quad*1i-b(j)).^2 - sigma^2*(2*u(j)*x_quad*1i-x_quad.^2)) );
% c = @(j, x_quad)( (b(j) - rho*sigma*x_quad*1i-d(j, x_quad))./(b(j) - rho*sigma*x_quad*1i+d(j, x_quad)));
% D = @(j, x_quad)( (b(j) - rho*sigma*x_quad*1i - d(j, x_quad))/sigma^2.* (1-exp(-d(j, x_quad)*T))./(1 - c(j, x_quad).*exp(-d(j, x_quad)*T)) );
% C = @(j, x_quad)( (r-q)*x_quad*1i*T + kappa*theta/sigma^2 * ( (b(j) - rho*sigma*x_quad*1i - d(j, x_quad))*T - 2*log((1-c(j, x_quad).*exp(-d(j, x_quad)*T))./(1-c(j, x_quad))) ) );
% f1_2 = @(j, x_quad)(exp(C(j, x_quad)+D(j, x_quad).*v+1i*x_quad.*log(S)));
% f = @(x_quad)( real(exp(-1i*x_quad*log(K))./(1i*x_quad).*(S.*exp(-q*T).*f1_2(1, x_quad) - K*exp(-r*T)*f1_2(2, x_quad))) );
% result_comb_call = 1/2*S(:,1)*exp(-q*T) - 1/2*K*exp(-r*T)+ 1/pi*sum(w_quad.*f(x_quad), 2);

%% o neco rychlejsi implementace
% d = @(j)( sqrt((rho*sigma*x_quad*1i-b(j)).^2 - sigma^2*(2*u(j)*x_quad*1i-x_quad.^2)) );
% c = @(j)( (b(j) - rho*sigma*x_quad*1i-d(j))./(b(j) - rho*sigma*x_quad*1i+d(j)));
% D = @(j)( (b(j) - rho*sigma*x_quad*1i - d(j))/sigma^2.* (1-exp(-d(j)*T))./(1 - c(j).*exp(-d(j)*T)) );
% C = @(j)( (r-q)*x_quad*1i*T + kappa*theta/sigma^2 * ( (b(j) - rho*sigma*x_quad*1i - d(j))*T - 2*log((1-c(j).*exp(-d(j)*T))./(1-c(j))) ) );
% f1_2 = @(j)(exp(C(j)+D(j).*v+1i*x_quad.*log(S)));
% f =  real(exp(-1i*x_quad*log(K))./(1i*x_quad).*(S.*exp(-q*T).*f1_2(1) - K*exp(-r*T)*f1_2(2))) ;
% result_comb_call = 1/2*S(:,1)*exp(-q*T) - 1/2*K*exp(-r*T)+ 1/pi*sum(w_quad.*f, 2);

rho_sigma_x_i = rho*sigma*x_quad*1i;
d1 = sqrt((rho_sigma_x_i-b(1)).^2 - sigma^2*(2*u(1)*x_quad*1i-x_quad.^2));
d2 = sqrt((rho_sigma_x_i-b(2)).^2 - sigma^2*(2*u(2)*x_quad*1i-x_quad.^2));
c1 =  (b(1) - rho_sigma_x_i-d1)./(b(1) - rho_sigma_x_i+d1);
c2 =  (b(2) - rho_sigma_x_i-d2)./(b(2) - rho_sigma_x_i+d2);
D1 = (b(1) - rho_sigma_x_i - d1)/sigma^2.* (1-exp(-d1*T))./(1 - c1.*exp(-d1*T));
D2 = (b(2) - rho_sigma_x_i - d2)/sigma^2.* (1-exp(-d2*T))./(1 - c2.*exp(-d2*T));
C1 =  (r-q)*x_quad*1i*T + kappa*theta/sigma^2 * ( (b(1) - rho_sigma_x_i - d1)*T - 2*log((1-c1.*exp(-d1*T))./(1-c1)) ) ;
C2 =  (r-q)*x_quad*1i*T + kappa*theta/sigma^2 * ( (b(2) - rho_sigma_x_i - d2)*T - 2*log((1-c2.*exp(-d2*T))./(1-c2)) ) ;
f1 = exp(C1+D1.*v+1i*x_quad.*log(S));
f2 = exp(C2+D2.*v+1i*x_quad.*log(S));
w_f =  w_quad.*real(exp(-1i*x_quad*log(K))./(1i*x_quad).*(S.*exp(-q*T).*f1 - K*exp(-r*T)*f2)) ;

result_comb_call = 1/2*S(:,1)*exp(-q*T) - 1/2*K*exp(-r*T)+ 1/pi*sum(w_f, 2);
result_comb_put = result_comb_call + K*exp(-r*T) - S(:,1)*exp(-q*T);

price = result_comb_call*(w == 1) + result_comb_put*(w == -1);
%% funkce primo vyuziva funkci
% nevyuziva funkci, ale data pro Gauss-Laguerre kvadraturu z wx_gauss_laguerre.mat. Data (x_quad, w_quad) z tohoto souboru je nutno naloadovat predem a pak poslat jako parametr do teto fce
end