function price = CDS_PDE_CIR_log(K, r, T, delta, gamma, kappa, theta, sigma, m, n)
% funkce ohodnocuje CDS pomoci PDE za predpokladu, ze stochasticka intenzita defaultu podkladovho jmena ma CIR dynamiku, v logaritmickem tvaru
% je to uplne stejna dynamika jako CIRu (vcetne potreby Fellerovy podminky), pouze vyjadrena v dx = d log(gamma) tvaru misto d (gamma), jak je "bezne"
% i parametry (kappa, theta, sigma) maji naprosto stejny vyznam jako v beznem CIRu
% funkce predpoklada, ze jsme v pozici PROTECTION SELLERA (tedy dostavame kupon K) a mame potencialni zavazek vyplatit LGD = (1 - delta). To lze snadno zmenit 'prehozenim znamenek' ve vyrazu RHS
% dale se predpoklada konstantni urokova sazba r
% tj. log intenzita x=ln(gamma) se ridi dx(t) = exp(-x(t))*(kappa*(theta - exp(x(t))) - 1/2*sigma^2)*dt + sqrt(exp(-x(t)))*sigma*dW(t)

%% vstupy
% K     = CDS kupon jako desetinne cislo, tj. napr. K = 0.02 pro 2 % kupon placeny v CDS jako premium
% r     = konstantni bezrizikova sazba s tenorem splatnosti dealu
% T     = residualni splatnost CDS dealu, jehoz hodnotu pocitame
% delta = recovery rate podkladu (toto cislo je typicky primo dane)
% gamma = jedna se o hodnotu intenzity v case oceneni, tedy gamma = gamma(0)
% kappa = parametr rychlosti mean reversion v CIR modelu
% theta = uroven navratove rychlosti promenne gamma, napr. 0.04
% sigma = parametr volatility
% m     = pocet spatial kroku intenzity
% n     = pocet casovych kroku

% volani: CDS_PDE_CIR_log(0.02, 0.05, 5, 0.4, 0.03, 0.3, 0.04, 0.12, 200, 50) 
% vrati hodnotu dealu z pohledu PROTECTION SELLERa, ktery konci za 5 let, dostavame kupon 0.02, urokova sazba je 0.05, recovery = 0.4, pocatecni intenzita gamma = 0.03
% kappa = 0.3, theta = 0.04, sigma = 0.12. Grid bude mit 200 bodu a casova osa bude mit 50 bodu

%% vypocet
x = log(gamma);
x_floor    = -14; % minimalni hodnota log intenzity na gridu, lze upravit
x_ceil     = 1; % maximalni hodnota log intenzity na gridu, lze upravit

dx = (x_ceil-x_floor)/(m+1);
x_grid = x_floor+dx:dx:x_ceil-dx;
dt = T/n; t = 0:dt:T;

A = get_A(x_grid, kappa, theta, sigma, r, dx); % matice koeficientu

V = zeros(m, n+1);
V(:, end) = 0; % CDS ma v case splatnosti hodnotu = 0, tedy g(gamma) v case T = 0, kde g(.) je fce finalni vyplaty
LHS_inv = inv(eye(m) - dt*0.5*A); % pomocny vypocet pro urychleni dalsich vypoctu
for i = numel(t)-1:-1:1
    RHS = (eye(m) + dt*0.5*A)*V(:, i+1) + dt*(-exp(x_grid(:)) * (1-delta) + K);  % "-" zde rika, ze jsme platce LGD pri defaultu, +K rika, ze dostavame CDS kupon K
                                                                                 % pokud bychom chteli zjistit hodnotu pozice protection buyer, tak nastavime vyraz dt*(exp(x_grid(:)) * (1-delta) - K)
    V(:, i) = LHS_inv*RHS;
end

price = interp1(x_grid, V(:, 1), x);

%% zde je pripadne moznost nastavit 3D graf
surf(t, x_grid, V); xlabel('time $(t)$', 'interpreter', 'latex'); ylabel('$x_2 = \ln \gamma$',  'interpreter', 'latex'); zlabel('$\tilde V(t,x_2)$',  'interpreter', 'latex');
colormap hsv; view(-126, 25); axis tight;

%% funkce primo vyuziva funkci 
% --

%% funkce neprimo vyuziva funkci 
% --
end

function A = get_A(x_grid, kappa, theta, sigma, r, dx)

[l, c, u, first_row, last_row] = get_lcu(x_grid, kappa, theta, sigma, r, dx);
A = diag(c) + diag(l(2:end), -1) + diag(u(1:end-1), 1);
% doplneni elementu popisujici derivace na hranicich x boundary
A(1, :) = first_row; % x = x_1 (dolni boundary)
A(end, :) = last_row; % x = x_m (horni boundary)

end

function [l, c, u, first_row, last_row] = get_lcu(x_grid, kappa, theta, sigma, r, dx)
m = numel(x_grid);
x = x_grid(:);

mu = exp(-x).*(kappa*(theta - exp(x)) - 0.5*sigma^2);
sigma_sq = exp(-x)*sigma^2;

l = -mu/(2*dx) + sigma_sq/(2*dx^2);
c = -sigma_sq/(dx^2) - (r + exp(x));
u = mu/(2*dx) + sigma_sq/(2*dx^2);
l(1) = 0; l(end) = 0;
c(1) = 0; c(end) = 0;
u(1) = 0; u(end) = 0;

% prvni radek matice A, tj LV_1(t) => derivace jsou aproximovany fwd diferenci s chybou O(dx^2)
% jedna se o koeficienty pro V_1, V_2, V_3, V_4
first_row = zeros(1, m);
first_row(1:4) = [-3/2*mu(1)/dx+sigma_sq(1)/dx^2-(r+exp(x(1))), 2*mu(1)/dx-5/2*sigma_sq(1)/dx^2, -mu(1)/(2*dx)+2*sigma_sq(1)/dx^2, -1/2*sigma_sq(1)/dx^2];

% posledni radek matice A, tj LV_m(t) => derivace jsou aproximovany bkw diferenci s chybou O(dx^2)
% jedna se o koeficienty pro V_{m-3}, V_{m-2}, V_{m-1}, V_{m}
last_row = zeros(1, m);
last_row(end-3:end) = [-1/2*sigma_sq(end)/dx^2, mu(end)/(2*dx)+2*sigma_sq(end)/dx^2, -2*mu(end)/dx-5/2*sigma_sq(end)/dx^2, 3/2*mu(end)/dx+sigma_sq(end)/dx^2-(r+exp(x(end)))];
end