function price = CDS_option_PDE_CIR_log(K, r, T, CDS_tenor, delta, gamma, kappa, theta, sigma, m, n)
% funkce pocita pomoci PDE hodnotu CALL opce na CDS s tenorem "tenor" a predpoklada, ze intenzita defaultu se ridi CIR procesem. PUT opci lze spocitat modifikaci funkce vyplaty
% u procesu intenzity CIRu se predpoklada log tvar, tj. x=log(gamma).
% je to uplne stejna dynamika jako CIRu (vcetne potreby Fellerovy podminky), pouze vyjadrena v dx = d log(gamma) tvaru misto d (gamma), jak je "bezne"
% i parametry (kappa, theta, sigma) maji naprosto stejny vyznam jako v beznem CIRu
% funkce si sama vola funkci pro log CIR model (CDS_PDE_CIR_log), protoze ten je lepsi z hlediska nizkych hodnot intenzity gamma nez bezny CIR model, pripadne tempered verze
% funkce predpoklada, ze je jen jedna urokova sazba r (konstantni) a to jak pro diskontovani opce na CDS, tak i pro CDS v podkladu samotne
% to asi neni uplne spravne, ale vysledek to prilis nezkresluje. Je mozne toto jeste vylepsit a nechat kazou sazbu jinou

%% vstupy
% K         = strike hodnota CDS opce
% r         = konstantni bezrizikova sazba s tenorem splatnosti dealu
% T         = splatnost CDS opce
% CDS_tenor = tenor podkladoveho CDS
% delta     = recovery rate CDS podkladu (toto cislo je typicky primo dane)
% gamma     = jedna se o hodnotu intenzity defaultu jmena v podkladu v case oceneni, tedy gamma = gamma(0)
% kappa     = parametr rychlosti mean reversion v CIR modelu
% theta     = uroven navratove rychlosti promenne gamma, napr 0.04
% sigma     = parametr volatility
% m         = pocet spatial kroku intenzity
% n         = pocet casovych kroku

% volani: CDS_option_PDE_CIR_log(0.01, 0.05, 2, 5, 0.4, 0.03, 0.3, 0.04, 0.1, 80, 40). Funkce oceni call opci na 5Y CDS, ktera maturuje za T = 2 roky. Opce ma strike K = 0.01. 
% Pokud nastane default jmena, na ktere je CDS vypsane, predpoklada se recovery delta = 0.4. Proces intenzity defaultu se ridi CIRem s pocatecni intenzitou gamma = 0.03 a parametry
% kappa = 0.3, theta = 0.04, sigma = 0.1. Grid bude mit 80 bodu a casova osa bude mit 40 bodu

%% vypocet
x = log(gamma);
x_floor    = -10; % minimalni hodnota log intenzity na gridu, lze upravit
x_ceil     = 0.3; % max hodnota log intenzity na gridu, lze upravit

dx = (x_ceil-x_floor)/(m+1);
x_grid = x_floor+dx:dx:x_ceil-dx;
dt = T/n; t = 0:dt:T;

A = get_A(x_grid, kappa, theta, sigma, r, dx);

V = zeros(m, n+1);
for j = 1:m
    %% zasadni cast oceneni, kdy zjistujeme, jake by byly v case T ferove CDS spready, kdyby gamma mela hodnotu gamma = exp(x_grid(j))
    % tyto CDS spready nam pak definuji vyplatu v case T na zaklade gammy = exp(x_grid(j))
    f = @(K_crit)(CDS_PDE_CIR_log(K_crit, r, CDS_tenor, delta, exp(x_grid(j)), kappa, theta, sigma, m+20, ceil((m+20)/2)));
    K_crit = fzero(f, 0.02);
    V(j, end) = max(K_crit-K, 0); % toto definuje finalni vyplatu call opce v case T
end

LHS_inv = inv(eye(m) - dt*0.5*A);
for i = numel(t)-1:-1:1
    RHS = (eye(m) + dt*0.5*A)*V(:, i+1);
    V(:, i) = LHS_inv*RHS;
end

price = interp1(x_grid, V(:, 1), x);

%% zde je pripadne moznost nastavit 3D graf
surf(t, x_grid, V); xlabel('time $(t)$', 'interpreter', 'latex'); ylabel('$x_2 = \ln \gamma$',  'interpreter', 'latex'); zlabel('$\tilde V(t,x_2)$',  'interpreter', 'latex');
colormap hsv; view(-126, 25); axis tight;
%% funkce primo vyuziva funkci 
% CDS_PDE_CIR_log(K, r, T, delta, gamma, kappa, theta, sigma, m, n)

%% funkce neprimo vyuziva funkci 
% --
end

function A = get_A(x_grid, kappa, theta, sigma, r, dx)

[l, c, u, first_row, last_row] = get_lcu(x_grid, kappa, theta, sigma, r, dx);
A = diag(c) + diag(l(2:end), -1) + diag(u(1:end-1), 1);
% doplneni elementu popisujici derivace na hranicich x boundary
A(1, :) = first_row; % x = x_1 (dolni boundary)
A(end, :) = last_row; % x = x_m (horni boundary)

end

function [l, c, u, first_row, last_row] = get_lcu(x_grid, kappa, theta, sigma, r, dx)
m = numel(x_grid);
x = x_grid(:);

mu = exp(-x).*(kappa*(theta - exp(x)) - 0.5*sigma^2);
sigma_sq = exp(-x)*sigma^2;

l = -mu/(2*dx) + sigma_sq/(2*dx^2);
c = -sigma_sq/(dx^2) - r;
u = mu/(2*dx) + sigma_sq/(2*dx^2);
l(1) = 0; l(end) = 0;
c(1) = 0; c(end) = 0;
u(1) = 0; u(end) = 0;

% prvni radek matice A, tj LV_1(t) => derivace jsou aproximovany fwd diferenci s chybou O(dx^2)
% jedna se o koeficienty pro V_1, V_2, V_3, V_4
first_row = zeros(1, m);
first_row(1:4) = [-3/2*mu(1)/dx+sigma_sq(1)/dx^2-r, 2*mu(1)/dx-5/2*sigma_sq(1)/dx^2, -mu(1)/(2*dx)+2*sigma_sq(1)/dx^2, -1/2*sigma_sq(1)/dx^2];

% posledni radek matice A, tj LV_m(t) => derivace jsou aproximovany bkw diferenci s chybou O(dx^2)
% jedna se o koeficienty pro V_{m-3}, V_{m-2}, V_{m-1}, V_{m}
last_row = zeros(1, m);
last_row(end-3:end) = [-1/2*sigma_sq(end)/dx^2, mu(end)/(2*dx)+2*sigma_sq(end)/dx^2, -2*mu(end)/dx-5/2*sigma_sq(end)/dx^2, 3/2*mu(end)/dx+sigma_sq(end)/dx^2-r];
end